@@ -5,6 +5,7 @@ Not intended to be used outside compas* packages.
 """
 import os
 import sys
+import subprocess
 
 PY3 = sys.version_info[0] == 3
 system = sys.platform
@@ -66,6 +67,18 @@ def absjoin(*parts):
     return os.path.abspath(os.path.join(*parts))
 
 
+# Cache whatever symlink function works (native or polyfill)
+_os_symlink = None
+
+
+def create_symlink_polyfill():
+    def symlink_ms(source, link_name):
+        subprocess.check_output(
+            ['mklink', '/D', link_name, source], stderr=subprocess.STDOUT, shell=True)
+
+    return symlink_ms
+
+
 def create_symlink(source, link_name):
     """Create a symbolic link pointing to source named link_name.
 
@@ -77,18 +90,26 @@ def create_symlink(source, link_name):
         This function is a polyfill of the native ``os.symlink``
         for Python 2.x on Windows platforms.
     """
-    os_symlink = getattr(os, 'symlink', None)
+    global _os_symlink
+    enable_retry_with_polyfill = False
 
-    if not callable(os_symlink) and os.name == 'nt':
-        import subprocess
+    if not _os_symlink:
+        _os_symlink = getattr(os, 'symlink', None)
 
-        def symlink_ms(source, link_name):
-            subprocess.check_output(
-                ['mklink', '/D', link_name, source], stderr=subprocess.STDOUT, shell=True)
+        if os.name == 'nt':
+            if not callable(_os_symlink):
+                _os_symlink = create_symlink_polyfill()
+            else:
+                enable_retry_with_polyfill = True
 
-        os_symlink = symlink_ms
+    try:
+        _os_symlink(source, link_name)
+    except OSError:
+        if not enable_retry_with_polyfill:
+            raise
 
-    os_symlink(source, link_name)
+        _os_symlink = create_symlink_polyfill()
+        _os_symlink(source, link_name)
 
 
 def remove_symlink(link):
