@@ -4,10 +4,15 @@ from __future__ import division
 
 import pickle
 import json
+import collections
 
 from copy import deepcopy
 from ast import literal_eval
 
+from math import pi
+
+from compas.utilities import average
+
 from compas.files import OBJ
 from compas.files import PLY
 from compas.files import STL
@@ -24,11 +29,16 @@ from compas.geometry import cross_vectors
 from compas.geometry import length_vector
 from compas.geometry import scale_vector
 from compas.geometry import add_vectors
+from compas.geometry import sum_vectors
 from compas.geometry import subtract_vectors
 from compas.geometry import normal_polygon
 from compas.geometry import area_polygon
 from compas.geometry import flatness
 from compas.geometry import Polyhedron
+from compas.geometry import angle_points
+from compas.geometry import bestfit_plane
+from compas.geometry import distance_point_plane
+from compas.geometry import distance_point_point
 
 from compas.datastructures import Datastructure
 
@@ -71,7 +81,6 @@ Mesh summary
 ================================================================================
 """
 
-
 class Mesh(FromToPickle,
            FromToJson,
            FromToData,
@@ -611,11 +620,12 @@ class Mesh(FromToPickle,
 
         Parameters
         ----------
-        vertices : list
-            A list of vertices, represented by their XYZ coordinates.
-        faces : list
-            A list of faces.
-            Each face is a list of indices referencing the list of vertex coordinates.
+        vertices : list, dict
+            A list of vertices, represented by their XYZ coordinates,
+            or a dictionary of vertex keys pointing to their XYZ coordinates.
+        faces : list, dict
+            A list of faces, represented by a list of indices referencing the list of vertex coordinates,
+            or a dictionary of face keys pointing to a list of indices referencing the list of vertex coordinates.
 
         Returns
         -------
@@ -636,11 +646,21 @@ class Mesh(FromToPickle,
 
         """
         mesh = cls()
-        for vkey, vertex in enumerate(vertices):
-            x, y, z = vertex
-            mesh.add_vertex(key=vkey, x=x, y=y, z=z)
-        for fkey, face in enumerate(faces):
-            mesh.add_face(face, fkey=fkey)
+
+        if isinstance(vertices, collections.Mapping):
+            for key, xyz in vertices.items():
+                mesh.add_vertex(key = key, attr_dict = {i: j for i, j in zip(['x', 'y', 'z'], xyz)})
+        else:
+            for x, y, z in iter(vertices):
+                mesh.add_vertex(x=x, y=y, z=z)
+
+        if isinstance(faces, collections.Mapping):
+            for fkey, vertices in faces.items():
+                mesh.add_face(vertices, fkey)
+        else:
+            for face in iter(faces):
+                mesh.add_face(face)
+
         return mesh
 
     @classmethod
@@ -2248,7 +2268,7 @@ class Mesh(FromToPickle,
         return self.face[fkey][i + 1]
 
     def face_adjacency_halfedge(self, f1, f2):
-        """Find the half-edge over which tow faces are adjacent.
+        """Find one half-edge over which two faces are adjacent.
 
         Parameters
         ----------
@@ -2274,6 +2294,27 @@ class Mesh(FromToPickle,
             if self.halfedge[v][u] == f2:
                 return u, v
 
+    def face_adjacency_vertices(self, f1, f2):
+        """Find all vertices over which two faces are adjacent.
+
+        Parameters
+        ----------
+        f1 : hashable
+            The identifier of the first face.
+        f2 : hashable
+            The identifier of the second face.
+
+        Returns
+        -------
+        list
+            The vertices separating face 1 from face 2.
+        None
+            If the faces are not adjacent.
+
+        """
+
+        return [vkey for vkey in self.face_vertices(f1) if vkey in self.face_vertices(f2)]
+
     def is_face_on_boundary(self, key):
         """Verify that a face is on a boundary.
 
@@ -2323,16 +2364,8 @@ class Mesh(FromToPickle,
         list
             The coordinates of the mesh centroid.
         """
-        A = 0
-        cx, cy, cz = 0.0, 0.0, 0.0
-        for fkey in self.faces():
-            a = self.face_area(fkey)
-            x, y, z = self.face_centroid(fkey)
-            cx += a * x
-            cy += a * y
-            cz += a * z
-            A += a
-        return [cx / A, cy / A, cz / A]
+
+        return scale_vector(sum_vectors([scale_vector(self.face_centroid(fkey), self.face_area(fkey)) for fkey in self.faces()]), 1. / self.area())
 
     def normal(self):
         """Calculate the average mesh normal.
@@ -2346,7 +2379,7 @@ class Mesh(FromToPickle,
             The coordinates of the mesh normal.
         """
 
-        return scale_vector(1. / self.area(), add_vectors([scale_vector(self.face_area(fkey), self.face_normal(fkey)) for fkey in self.faces()]))
+        return scale_vector(sum_vectors([scale_vector(self.face_normal(fkey), self.face_area(fkey)) for fkey in self.faces()]), 1. / self.area())
 
     # --------------------------------------------------------------------------
     # vertex geometry
@@ -2482,6 +2515,27 @@ class Mesh(FromToPickle,
         vectors = [self.face_normal(fkey, False) for fkey in self.vertex_faces(key) if fkey is not None]
         return normalize_vector(centroid_points(vectors))
 
+    def vertex_curvature(self, vkey):
+        """Dimensionless vertex curvature.
+
+        Parameters
+        ----------
+        fkey : Key
+            The face key.
+
+        Returns
+        -------
+        float
+            The dimensionless curvature.
+
+        References
+        ----------
+        .. [1] Botsch, Mario, et al. *Polygon mesh processing.* AK Peters/CRC Press, 2010.
+
+        """
+
+        return 2 * pi - sum([angle_points(mesh.vertex_coordinates(vkey), mesh.vertex_coordinates(u), mesh.vertex_coordinates(v)) for u, v in pairwise(self.vertex_neighbors(vkey, ordered = True) + self.vertex_neighbors(vkey, ordered = True)[:1])])
+
     # --------------------------------------------------------------------------
     # edge geometry
     # --------------------------------------------------------------------------
@@ -2607,6 +2661,78 @@ class Mesh(FromToPickle,
         face = range(len(self.face_vertices(fkey)))
         return flatness(vertices, [face])[0]
 
+    def face_aspect_ratio(self, fkey):
+        """Face aspect ratio as the ratio between the lengths of the maximum and minimum face edges.
+
+        Parameters
+        ----------
+        fkey : Key
+            The face key.
+
+        Returns
+        -------
+        float
+            The aspect ratio.
+
+         References
+        ----------
+        .. [1] Wikipedia. *Types of mesh*.
+               Available at: https://en.wikipedia.org/wiki/Types_of_mesh.
+
+        """
+
+        face_edge_lengths = [self.edge_length(u, v) for u, v in self.face_halfedges(fkey)]
+        return max(face_edge_lengths) / min(face_edge_lengths)
+
+    def face_skewness(self, fkey):
+        """Face skewness as the maximum absolute angular deviation from the ideal polygon angle.
+
+        Parameters
+        ----------
+        fkey : Key
+            The face key.
+
+        Returns
+        -------
+        float
+            The skewness.
+
+         References
+        ----------
+        .. [1] Wikipedia. *Types of mesh*.
+               Available at: https://en.wikipedia.org/wiki/Types_of_mesh.
+
+        """
+
+        ideal_angle = 180 * (1 - 2 / float(len(self.face_vertices(fkey))))
+
+        angles = [angle_points(self.vertex_coordinates(v), self.vertex_coordinates(u), self.vertex_coordinates(w), deg = True) for u, v, w in window(self.face_vertices(fkey) + self.face_vertices(fkey)[:2], n = 3)]
+
+        return max((max(angles) - ideal_angle) / (180 - ideal_angle), (ideal_angle - min(angles)) / ideal_angle)
+
+    def face_curvature(self, fkey):
+        """Dimensionless face curvature as the maximum face vertex deviation from the best-fit plane of the face vertices divided by the average lengths of the face vertices to the face centroid.
+
+        Parameters
+        ----------
+        fkey : Key
+            The face key.
+
+        Returns
+        -------
+        float
+            The dimensionless curvature.
+
+        """
+
+        plane = bestfit_plane([self.vertex_coordinates(vkey) for vkey in self.vertices()])
+
+        max_deviation = max([distance_point_plane(self.vertex_coordinates(vkey), plane) for vkey in self.vertices()])
+
+        average_distances = average([distance_point_point(self.vertex_coordinates(vkey), self.face_centroid(fkey)) for vkey in self.vertices()])
+
+        return max_deviation / average_distances
+
     # def face_circle(self, fkey):
     #     pass
 
@@ -3076,19 +3202,19 @@ if __name__ == '__main__':
 
     # # mesh.add_face([a, b, c, d, e, f, g, h])
 
-    for k in mesh.faces():
-        print(k, mesh.is_face_on_boundary(k))
+    # for k in mesh.faces():
+    #     print(k, mesh.is_face_on_boundary(k))
 
 
-    print(list(mesh.edges(True)))
+    # print(list(mesh.edges(True)))
 
 
-    plotter = MeshPlotter(mesh)
+    # plotter = MeshPlotter(mesh)
 
-    plotter.draw_vertices()
-    plotter.draw_edges()
-    plotter.draw_faces(text='key')
-    plotter.show()
+    # plotter.draw_vertices()
+    # plotter.draw_edges()
+    # plotter.draw_faces(text='key')
+    # plotter.show()
 
     # print(mesh.get_vertices_attribute('x'))
     # print(mesh.get_vertices_attributes('xy'))
@@ -3096,4 +3222,42 @@ if __name__ == '__main__':
     # print(mesh.get_edges_attribute('q', 1.0))
     # print(mesh.get_edges_attributes('qf', (1.0, 2.0)))
 
+    vertices = {
+        0: [0, 0, 0],
+        1: [1, 1, 0],
+        2: [1, -1, 0],
+        3: [-1, -1, 0],
+        18: [-1, 1, 0]
+    }
+    faces = {
+        0: [0, 2, 1],
+        45: [0, 18, 3]
+    }
 
+    mesh = Mesh.from_vertices_and_faces(vertices, faces)
+
+    plotter = MeshPlotter(mesh)
+    plotter.draw_vertices(text='key')
+    plotter.draw_edges()
+    plotter.draw_faces(text='key')
+    plotter.show()
+
+    vertices = [
+        [0, 0, 0],
+        [1, 1, 0],
+        [1, -1, 0],
+        [-1, -1, 0],
+        [-1, 1, 0]
+    ]
+    faces = [
+        [0, 2, 1],
+        [0, 4, 3]
+    ]
+
+    mesh = Mesh.from_vertices_and_faces(vertices, faces)
+
+    plotter = MeshPlotter(mesh)
+    plotter.draw_vertices(text='key')
+    plotter.draw_edges()
+    plotter.draw_faces(text='key')
+    plotter.show()
