@@ -1,11 +1,58 @@
 """
 ********************************************************************************
-compas.geometry
+geometry
 ********************************************************************************
 
 .. currentmodule:: compas.geometry
 
 
+Classes
+=======
+
+Primitives
+----------
+
+.. autosummary::
+    :toctree: generated/
+    :nosignatures:
+
+    Vector
+    Point
+    Line
+    Polyline
+    Polyhedron
+    Polygon
+    Plane
+    Frame
+    Circle
+    Box
+    Bezier
+
+XForms
+------
+
+.. autosummary::
+    :toctree: generated/
+    :nosignatures:
+
+    Transformation
+    Rotation
+    Translation
+    Scale
+    Reflection
+    Projection
+    Shear
+
+Other
+-----
+
+.. autosummary::
+    :toctree: generated/
+    :nosignatures:
+
+    KDTree
+
+
 Functions
 =========
 
@@ -86,8 +133,10 @@ Bounding Box
     :toctree: generated/
     :nosignatures:
 
-    bbox
-    bbox_numpy
+    bounding_box
+    bounding_box_xy
+    oriented_bounding_box_numpy
+    oriented_bounding_box_xy_numpy
 
 Bestfit
 -------
@@ -344,53 +393,10 @@ Triangulation
 
     delaunay_from_points
     delaunay_from_points_numpy
+    delaunay_triangle
+    constrained_delaunay_triangle
+    conforming_delaunay_triangle
     voronoi_from_points_numpy
-    voronoi_from_delaunay
-
-
-Classes
-=======
-
-Primitives
-----------
-
-.. autosummary::
-    :toctree: generated/
-    :nosignatures:
-
-    Vector
-    Point
-    Line
-    Polyline
-    Polygon
-    Plane
-    Frame
-    Circle
-    Box
-
-XForms
-------
-
-.. autosummary::
-    :toctree: generated/
-    :nosignatures:
-
-    Transformation
-    Rotation
-    Translation
-    Scale
-    Reflection
-    Projection
-    Shear
-
-Other
------
-
-.. autosummary::
-    :toctree: generated/
-    :nosignatures:
-
-    KDTree
 
 
 """
