@@ -7,12 +7,6 @@ from math import fabs
 from random import sample
 
 
-__author__    = ['Tom Van Mele', ]
-__copyright__ = 'Copyright 2016 - Block Research Group, ETH Zurich'
-__license__   = 'MIT License'
-__email__     = 'vanmelet@ethz.ch'
-
-
 __all__ = [
     'allclose',
     'add_vectors',
@@ -998,6 +992,8 @@ def multiply_matrices(A, B):
     [[4.0, 0.0, 0.0], [0.0, 4.0, 0.0], [0.0, 0.0, 4.0]]
 
     """
+    A = list(A)
+    B = list(B)
     n = len(B)     # number of rows in B
     o = len(B[0])  # number of cols in B
     if not all(len(row) == o for row in B):
