@@ -1,6 +1,6 @@
 """
 ********************************************************************************
-compas.plotters
+plotters
 ********************************************************************************
 
 .. currentmodule:: compas.plotters
