@@ -5,8 +5,10 @@ from __future__ import division
 import os
 import sys
 import json
+import tempfile
 
 import compas
+import compas._os
 
 from compas.utilities import DataEncoder
 from compas.utilities import DataDecoder
@@ -233,9 +235,10 @@ class XFunc(object):
 
     """
 
-    def __init__(self, funcname, basedir='.', tmpdir='.', delete_files=True,
-                 verbose=True, callback=None, callback_args=None, python='pythonw',
-                 paths=None, serializer='json'):
+    def __init__(self, funcname, basedir='.', tmpdir=None, delete_files=True,
+                 verbose=True, callback=None, callback_args=None, python=None,
+                 paths=None, serializer='json',
+                 argtypes=None, kwargtypes=None, restypes=None):
         self._basedir       = None
         self._tmpdir        = None
         self._callback      = None
@@ -243,14 +246,17 @@ class XFunc(object):
         self._serializer    = None
         self.funcname       = funcname
         self.basedir        = basedir
-        self.tmpdir         = tmpdir
+        self.tmpdir         = tmpdir or tempfile.mkdtemp('compas_xfunc')
         self.delete_files   = delete_files
         self.verbose        = verbose
         self.callback       = callback
         self.callback_args  = callback_args
-        self.python         = python
+        self.python         = compas._os.select_python(python)
         self.paths          = paths or []
         self.serializer     = serializer
+        self.argtypes       = argtypes
+        self.kwargtypes     = kwargtypes
+        self.restypes       = restypes
         self.data           = None
         self.profile        = None
         self.error          = None
@@ -336,14 +342,25 @@ class XFunc(object):
             In this case, check the ``error`` attribute for more information.
 
         """
-        idict = {'args': args, 'kwargs': kwargs}
+        # if self.argtypes:
+        #     args = [arg for arg in args]
+
+        # if self.kwargtypes:
+        #     kwargs = {name: value for name, value in kwargs.items()}
+
+        idict = {
+            'args': args,
+            'kwargs': kwargs,
+            # 'argtypes': self.argtypes,
+            # 'kwargtypes': self.kwargtypes,
+            # 'restypes': self.restypes
+        }
 
         if self.serializer == 'json':
             with open(self.ipath, 'w+') as fo:
                 json.dump(idict, fo, cls=DataEncoder)
         else:
             with open(self.ipath, 'wb+') as fo:
-                # pickle.dump(idict, fo, protocol=pickle.HIGHEST_PROTOCOL)
                 pickle.dump(idict, fo, protocol=2)
 
         with open(self.opath, 'w+') as fh:
@@ -401,36 +418,76 @@ class XFunc(object):
 
 if __name__ == '__main__':
 
+    import random
+
     import compas
-
     from compas.datastructures import Mesh
-    # from compas.plotters import MeshPlotter
     from compas.utilities import XFunc
+    from compas_rhino.artists import MeshArtist
 
-    fd_numpy = XFunc('compas.numerical.fd.fd_numpy.fd_numpy', delete_files=True, serializer='json')
+    dr = XFunc('compas.numerical.dr_numpy')
+
+    dva = {
+        'is_fixed': False,
+        'x': 0.0,
+        'y': 0.0,
+        'z': 0.0,
+        'px': 0.0,
+        'py': 0.0,
+        'pz': 0.0,
+        'rx': 0.0,
+        'ry': 0.0,
+        'rz': 0.0,
+    }
+
+    dea = {
+        'qpre': 1.0,
+        'fpre': 0.0,
+        'lpre': 0.0,
+        'linit': 0.0,
+        'E': 0.0,
+        'radius': 0.0,
+    }
 
     mesh = Mesh.from_obj(compas.get('faces.obj'))
 
-    vertices = mesh.get_vertices_attributes('xyz')
-    edges    = list(mesh.edges())
-    fixed    = list([key for key in mesh.vertices() if mesh.vertex_degree(key) == 2])
-    q        = mesh.get_edges_attribute('q', 1.0)
-    loads    = mesh.get_vertices_attributes(('px', 'py', 'pz'), (0.0, 0.0, 0.0))
-
-    xyz, q, f, l, r = fd_numpy(vertices, edges, fixed, q, loads)
-
-    print(type(xyz))
+    mesh.update_default_vertex_attributes(dva)
+    mesh.update_default_edge_attributes(dea)
 
     for key, attr in mesh.vertices(True):
-       attr['x'] = xyz[key][0]
-       attr['y'] = xyz[key][1]
-       attr['z'] = xyz[key][2]
+        attr['is_fixed'] = mesh.vertex_degree(key) == 2
 
-    # plotter = MeshPlotter(mesh)
-    # plotter.draw_vertices()
-    # plotter.draw_faces()
-    # plotter.draw_edges()
-    # plotter.show()
+    for u, v, attr in mesh.edges(True):
+        attr['qpre'] = 1.0 * random.randint(1, 7)
 
-    print(fd_numpy.profile)
+    k_i = mesh.key_index()
+
+    vertices = mesh.get_vertices_attributes(('x', 'y', 'z'))
+    edges    = [(k_i[u], k_i[v]) for u, v in mesh.edges()]
+    fixed    = [k_i[key] for key in mesh.vertices_where({'is_fixed': True})]
+    loads    = mesh.get_vertices_attributes(('px', 'py', 'pz'))
+    qpre     = mesh.get_edges_attribute('qpre')
+    fpre     = mesh.get_edges_attribute('fpre')
+    lpre     = mesh.get_edges_attribute('lpre')
+    linit    = mesh.get_edges_attribute('linit')
+    E        = mesh.get_edges_attribute('E')
+    radius   = mesh.get_edges_attribute('radius')
+
+    xyz, q, f, l, r = dr(vertices, edges, fixed, loads, qpre, fpre, lpre, linit, E, radius, kmax=100)
+
+    for key, attr in mesh.vertices(True):
+        index = k_i[key]
+        attr['x'] = xyz[index][0]
+        attr['y'] = xyz[index][1]
+        attr['z'] = xyz[index][2]
+
+    artist = MeshArtist(mesh, layer="XFunc::Mesh")
+
+    artist.clear_layer()
+
+    artist.draw_vertices()
+    artist.draw_edges()
+    artist.draw_faces()
+
+    artist.redraw()
 
